@@ -4,7 +4,7 @@ use warnings;
 use Carp;
 use Data::Dumper;
 use Scalar::Util qw( weaken );
-our $VERSION = '2.2.0';
+our $VERSION = '2.2.1';
 
 my $GlobalRef = {};
 
@@ -350,7 +350,6 @@ sub push_perl {
         }
 
         elsif ( $ref eq 'ARRAY' ) {
-            $self->dump();
             my $arr_idx = $self->push_array();
             $stash->{$val} = $self->get_heapptr(-1);
             my $len = scalar @{$val};
@@ -980,6 +979,8 @@ package JavaScript::Duktape::Util;
 1;
 
 __END__
+=encoding utf-8
+
 =head1 NAME
 
 JavaScript::Duktape - Perl interface to Duktape embeddable javascript engine
@@ -1033,7 +1034,7 @@ setting number below 256k will croak.
     max_memory => 256 * 1024 * 2
 
 You can resize the memory allowed to consume on different executions by calling
-C<resize_memory> method, see L<Sandboxing> section below.
+C<resize_memory> method, see L</Sandboxing> section below.
 
 =item timout
 
@@ -1048,7 +1049,7 @@ You can override this value later on another code evaluation by calling C<set_ti
 
     $js->set_timeout(25);
 
-See L<Sandboxing> section below
+See L</Sandboxing> section below
 
 =back
 
@@ -1365,6 +1366,12 @@ See C<examples/this.pl>
 
 =back
 
+=head1 DEAFULT JAVASCRIPT FUNCTIONS
+
+JavaScript::Duktape has C<alert> and C<print> functions available as global
+functions to javascript, where C<alert> prints to STDERR and C<print> prints
+to STDOUT.
+
 =head1 CAVEATS
 
 =head2 VM methods
